@@ -1,4 +1,5 @@
 -- tested with postgresql
+-- you can add a unique sequence id to the table if you want
 
 drop table linksdb;
 
@@ -10,10 +11,14 @@ create table linksdb (
     errorstring    varchar(50),
     validstring    varchar(50),
     warningstring  varchar(50),
-    infoString     varchar(150),
+    infostring     varchar(150),
     valid          int,
     url            varchar(50),
     line           int,
+    col            int,
     name           varchar(50),
+    checktime      int,
+    dltime         int,
+    dlsize         int,
     cached         int
 );
