@@ -1,13 +1,14 @@
+# -*- coding: iso-8859-1 -*-
 import os, linkcheck
 config = linkcheck.Config.Configuration()
 config.addLogger('test', linkcheck.test_support.TestLogger)
 config['recursionlevel'] = 1
 config['log'] = config.newLogger('test')
-config["anchors"] = 1
-config["verbose"] = 1
-config.disableThreading()
+config["anchors"] = True
+config["verbose"] = True
+config.setThreads(0)
 htmldir = "test/html"
-for file in ('http.html',):
-    url = os.path.join(htmldir, file)
+for filename in ('http.html',):
+    url = os.path.join("test", "html", filename)
     config.appendUrl(linkcheck.UrlData.GetUrlDataFrom(url, 0, config))
 linkcheck.checkUrls(config)
